/* @(#)59  1.7  include/m6attrs.h, m6co_headers, sco_3.0, 301FCS 9/9/94 13:05:03, SecureWare, Inc. */

/*
 * Copyright (c) 1990-1994 SecureWare, Inc.  All rights reserved.
 *
 * m6attrs.h
 *
 * USER INCLUDE FILE
 *
 * This file contains external definitions for the libm6 library.
 *
 */

/*
 * MaxSix 2.1, July 27, 1992
 * Copyright 1990-1992, SecureWare, Inc.
 */

#ifndef __M6ATTRS__ /*{*/
#define __M6ATTRS__

#include <sys/secdefines.h>

#if SEC_MSIX_IPC /*{*/

#include <sys/sctnmasks.h>

/* 
 * Declare the MaxSix extended errno return value.  If you call m6ext_err(),
 * m6errno will be set by the kernel to the MaxSix extended error return
 * code.  These values are found in sys/sctnerrno.h
 *
 * Note that a system running on a SecureWare OS actually uses sec_errno
 * rather than a separate m6errno.  This keeps all of the security
 * related errors in a single variable.
 */
#if SEC_BASE	/* i.e. this is a SecureWare operating system! */
#define m6errno	sec_errno
#define m6ext_err()	getsecerrno()
#define pm6error(s)	psecerror(s)
#else
int m6errno;
#endif

/*
 * Types and defines for the m6attr_ control structure and accessing it.
 */
typedef void **m6attr_t;
#define M6_ATTR_T_END	-2

/*
 * Constants used as parameters to the library functions.
 */
#define M6_EXT_ATTRS_ON		1
#define M6_EXT_ATTRS_OFF	0
#define M6_NEW_ATTRS_ON		1
#define M6_NEW_ATTRS_OFF	0
#define M6_TCB_ON		1
#define M6_TCB_OFF		0
#define	M6_CHANGE_ALL_ATTRS	-1
#define	M6_USE_DEFAULT_ATTRS	0

/*
 * Resolve differences between SecureWare's 2.0 and 3.0 calling conventions.
 */
#if SEC_2_0
#define MAND_TAG_TO_IR(mtag, mir)	mand_tag_to_ir(&(mtag), (mir))
#define MACILB_TAG_TO_IR(mtag, mir, mwhich)	\
	macilb_tag_to_ir(&(mtag), (mir), (mwhich))
#define NCAV_TAG_TO_IR(ntag, nir)	ncav_tag_to_ir(&(ntag), (nir))
#define ACL_TAG_TO_IR(atag, air)	acl_tag_to_ir(&(atag), (air))
#else
#define MAND_TAG_TO_IR(mtag, mir)	mand_tag_to_ir((mtag), (mir))
#define MACILB_TAG_TO_IR(mtag, mir, mwhich)	\
	macilb_tag_to_ir((mtag), (mir), (mwhich))
#define NCAV_TAG_TO_IR(ntag, nir)	ncav_tag_to_ir((ntag), (nir))
#define ACL_TAG_TO_IR(atag, air)	acl_tag_to_ir((atag), (air))
#endif


/***********************************************************
 * Macros used for accessing parts of the m6attr_t structure
 ***********************************************************/
#if SEC_MAC
#define m6_sl(a)	( * (mand_ir_t **)	& (a)[M6_SEN_LABEL]	)
#define M6_SL_WILDCARD	(M6M_SEN_LABEL<<16)
#endif
#if SEC_ILB
#define m6_il(a)	( * (ilb_ir_t **)	& (a)[M6_INFO_LABEL]	)
#define M6_IL_WILDCARD	(M6M_INFO_LABEL<<16)
#endif
#if SEC_NCAV
#define m6_ncav(a)	( * (ncav_ir_t **)	& (a)[M6_NAT_CAVEATS]	)
#define M6_NCAV_WILDCARD	(M6M_NAT_CAVEATS<<16)
#endif
#if SEC_ACL
#define m6_acl(a)	( * (acle_t **)		& (a)[M6_ACL]		)
#define M6_ACL_WILDCARD	(M6M_ACL<<16)
#endif
#define m6_priv(a)	( * (priv_t **)		& (a)[M6_PRIVILEGES]	)
#if SEC_PRIV
#define m6_gpriv(a)	( * (priv_t **)		& (a)[M6_GPRIVILEGES]	)
#endif
#define m6_luid(a)	( * (uid_t *)		 ((a)[M6_LUID])		)
#define m6_luid_ptr(a)	( * (uid_t **)		& (a)[M6_LUID]		)
#define m6_uid(a)	( * (uid_t *)		 ((a)[M6_UID])		)
#define m6_uid_ptr(a)	( * (uid_t **)		& (a)[M6_UID]		)
#define m6_gid(a)	( * (gid_t *)		 ((a)[M6_GID])		)
#define m6_gid_ptr(a)	( * (gid_t **)		& (a)[M6_GID]		)
#define m6_grp(a)	( * (gid_t **)		& (a)[M6_GROUPS]	)
#define m6_pid(a)	( * (pid_t *)		 ((a)[M6_PID])		)
#define m6_pid_ptr(a)	( * (pid_t **)		& (a)[M6_PID]		)
#define m6_sid(a)	( * (u_long *)		 ((a)[M6_SESSION_ID])	)
#define m6_sid_ptr(a)	( * (u_long **)		& (a)[M6_SESSION_ID]	)


#if SEC_MSIX_STREAMS
#include <sys/stropts.h>

/**************************************
 * Stream and TLI specific definitions.
 *************************************/
/* System call numbers for getmsg_attr() and putmsg_attr() */
#define GETMSG_ATTR	90
#define PUTMSG_ATTR	91

/*
 * Used in our pseudo-tli calls, t_rcvudat_attr() and t_sndudat_attr().
 */
struct t_unitdatattr {
	struct strbuf addr;		/*  address           */
	struct strbuf opt;		/* options            */
	struct strbuf udata;		/* user data          */
	struct strbuf attr;		/* security attributes. */
};
#endif


/**********************
 * Function prototypes.
 **********************/
int size_m6attr();
m6attr_t m6attr_alloc();
int m6build_attr(/* attr, ir_ptr_ptr */);
void m6copy_attr(/* attr1, attr2 */);
int m6create_attr_buf(/* ctl, mask, num_acls, buf_addr */);
int m6def_attr(/* fd, change_mask, def_mask, attr_ptr */);
int m6exception(/* fd, flags, err_code, net_err, sid */);
int m6ext_attr(/* fd, cmd */);
int m6ext_err();
int m6last_attr(/* fd, attr_ptr, attr_mask */);
int m6new_attr(/* fd, cmd */);
int m6peek(/* fd, attr_ptr, attr_mask */);
int m6recv(/* fd, data_ptr, data_len, addr_ptr, addr_len, attr_ptr,new_attrs*/);
int m6recv_attr(/* fd, data_ptr, data_len, addr_ptr, addr_len, attr, oob */);
int m6recv_mask(/* fd, change_mask, receive_mask */);
int m6recv_oob(/* fd, data_ptr, data_len, attr_ptr, new_attrs */);
int size_sec_attr(/* num_acls */);
struct m6_sec_attr *m6ACL_sec_attr_alloc(/* num_acls */);
struct m6_sec_attr *m6sec_attr_alloc();
int m6send(/* fd, data_ptr, data_len, addr_ptr, addr_len, attr_ptr */);
int m6send_attr(/* fd, data_ptr, data_len, addr_ptr, addr_len, attr_ptr */);
int m6send_oob(/* fd, data_ptr, data_len, addr_ptr, addr_len, attr_ptr */);
void m6set_attr(/* attr1, attr2, num_acls, ptr */);
#if SEC_MSIX_STREAMS
int getmsg_attr(/* fdes, ctl, data, attr, flags */);
int putmsg_attr(/* fdes, ctl, data, attr, flags */);
#endif
#if SCO && !SEC_MSIX_STREAMS
int sendmsg(/* sock, msg, flags */);
int recvmsg(/* sock, msg, flags */);
#endif
int m6tcb(/* fd, cmd */);
u_long m6unload_attr(/* attr, ir_ptr_ptr */);
#if SEC_MSIX_STREAMS && SEC_MSIX_IPC
int t_rcvattr(/* fd, buf, nbytes, attr, flags */);
int t_rcvudat_attr(/* fd, unitdata, flags */);
int t_sndattr(/* fd, buf, nbytes, attr, flags */);
int t_sndudat_attr(/* fd, unitdata */);
#endif
int m6map_token(/* direction, in_mask, out_mask, attr, tokens */);
int m6xlat_token(/* rhost, mask, in, out */);
int m6token_map(/* mask, src, dest */);
int m6attr_map(/* mask, src, dest, out_mask */);

#endif /* SEC_MSIX_IPC }*/
#endif /* __M6ATTRS__ }*/
